/////////////////////////////////////////////////////////////////////////
//                                                                     // 
//                     m68882real                                      //
//                    ~~~~~~~~~~~~                                     //
// C++ class enabling direct use of the m68882's 96-bit representation //
//                                                                     //
//                                                                     //
//                Written AUG98 by R.C.Lafferty                        //
//  BSc (hons) Computing Student at Farnborough College of Technology  //
//                                                                     //
// (Re)liability:                                                      //
// ~~~~~~~~~~~~~~                                                      //
// This is a purely experimental class and while it has been tested    //
// for basic correctness, it is only to be used as an educational      //
// example. However, the code provided here may indeed be used by      //
// coders who have fully tested it themselves and who use it at their  //
// own risk. Feel free to improve and re-distribute it.                //
//                                                                     //
// Feedback:                                                           //
// ~~~~~~~~~                                                           //
// Comments may be sent to: 10026254@farn-ct.ac.uk                     //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

class m68882real
{
private:

    unsigned long num[12/sizeof(unsigned long)]; 

 public:

    m68882real(void);             

    m68882real(double);      
    m68882real(const m68882real &);  
    m68882real &operator=(const m68882real &);
   ~m68882real();  

    operator double()const;    

    m68882real operator-(void)const;   
    m68882real operator+(void)const;

    m68882real &operator--(void);   
    m68882real &operator++(void);
    m68882real operator--(int);
    m68882real operator++(int);

    m68882real &operator+=(const m68882real &);
    m68882real &operator-=(const m68882real &);
    m68882real &operator*=(const m68882real &);
    m68882real &operator/=(const m68882real &);

    m68882real operator+(const m68882real &)const;
    m68882real operator-(const m68882real &)const;
    m68882real operator*(const m68882real &)const;
    m68882real operator/(const m68882real &)const;

    int operator!(void)const;

    int operator==(const m68882real &)const;
    int operator!=(const m68882real &)const;
    int operator< (const m68882real &)const;
    int operator<=(const m68882real &)const;
    int operator> (const m68882real &)const;
    int operator>=(const m68882real &)const;

    m68882real &operator=(double); 

    friend m68882real   sin(const m68882real &);
    friend m68882real   cos(const m68882real &);
    friend m68882real   tan(const m68882real &);
    friend m68882real  asin(const m68882real &);
    friend m68882real  acos(const m68882real &);
    friend m68882real  atan(const m68882real &);
    friend m68882real  sinh(const m68882real &);
    friend m68882real  cosh(const m68882real &);
    friend m68882real  tanh(const m68882real &);
    friend m68882real   exp(const m68882real &);
    friend m68882real   log(const m68882real &);
    friend m68882real log10(const m68882real &);
    friend m68882real  sqrt(const m68882real &);
    friend m68882real  ceil(const m68882real &);
    friend m68882real floor(const m68882real &);
    friend m68882real  fabs(const m68882real &);
    friend m68882real  fmod(const m68882real &,const m68882real &);
    friend m68882real  modf(const m68882real &,m68882real *);
    
    // friend m68882real pow(const m68882real &a,const m68882real &b);
    // friend m68882real ldexp(const m68882real &);
    // friend m68882real frexp(const m68882real &);
};

m68882real   sin(const m68882real &);
m68882real   cos(const m68882real &);
m68882real   tan(const m68882real &);
m68882real  asin(const m68882real &);
m68882real  acos(const m68882real &);
m68882real  atan(const m68882real &);
m68882real  sinh(const m68882real &);
m68882real  cosh(const m68882real &);
m68882real  tanh(const m68882real &);
m68882real   exp(const m68882real &);
m68882real   log(const m68882real &);
m68882real log10(const m68882real &);
m68882real  sqrt(const m68882real &);
m68882real  ceil(const m68882real &);
m68882real floor(const m68882real &);
m68882real  fabs(const m68882real &);
m68882real  fmod(const m68882real &,const m68882real &);
m68882real  modf(const m68882real &,m68882real *);

// m68882real pow(const m68882real &a,const m68882real &b);
// m68882real ldexp(const m68882real &);
// m68882real frexp(const m68882real &);


inline m68882real operator+(const m68882real &r,double d){return r+m68882real(d);}
inline m68882real operator-(const m68882real &r,double d){return r-m68882real(d);}
inline m68882real operator*(const m68882real &r,double d){return r*m68882real(d);}
inline m68882real operator/(const m68882real &r,double d){return r/m68882real(d);}

inline m68882real operator+(double d,const m68882real &r){return m68882real(d)+r;}
inline m68882real operator-(double d,const m68882real &r){return m68882real(d)-r;}
inline m68882real operator*(double d,const m68882real &r){return m68882real(d)*r;}
inline m68882real operator/(double d,const m68882real &r){return m68882real(d)/r;}

inline int operator==(const m68882real &r,double d){return r==m68882real(d);}
inline int operator!=(const m68882real &r,double d){return r!=m68882real(d);}
inline int operator< (const m68882real &r,double d){return r< m68882real(d);}
inline int operator<=(const m68882real &r,double d){return r<=m68882real(d);}
inline int operator> (const m68882real &r,double d){return r> m68882real(d);}
inline int operator>=(const m68882real &r,double d){return r>=m68882real(d);}

inline int operator==(double d,const m68882real &r){return m68882real(d)==r;}
inline int operator!=(double d,const m68882real &r){return m68882real(d)!=r;}
inline int operator< (double d,const m68882real &r){return m68882real(d)< r;}
inline int operator<=(double d,const m68882real &r){return m68882real(d)<=r;}
inline int operator> (double d,const m68882real &r){return m68882real(d)> r;}
inline int operator>=(double d,const m68882real &r){return m68882real(d)>=r;}


